"""
A class for editing an asset object
(C) Copyright 2005 Hamster Republic Productions
"""

########################################################################
from motherhamster import cheekpouches

import safe_wxversion
import wx
import wx.lib.scrolledpanel
import math
import pygame

from motherhamster.mathobj import vec2d

########################################################################
def _friendly_class_name(class_object):
    return repr(class_object).strip("<'>").split('.')[-1].split(' at ')[0]

########################################################################
def _create_property_editor(parent, asset, key, property, writable):
    if isinstance(property, cheekpouches.AssetRef):
        return AssetRefEditor(parent, asset, key, property, writable)
    elif isinstance(property, cheekpouches.AssetEnum):
        return AssetEnumEditor(parent, asset, key, property, writable)
    elif isinstance(property, cheekpouches.AssetInteger):
        return AssetNumberEditor(parent, asset, key, property, writable, allow_decimal = False)
    elif isinstance(property, cheekpouches.AssetFloat):
        return AssetNumberEditor(parent, asset, key, property, writable, allow_decimal = True)
    elif isinstance(property, cheekpouches.AssetString):
        return AssetStringEditor(parent, asset, key, property, writable)
    elif isinstance(property, cheekpouches.AssetFlag):
        return AssetFlagEditor(parent, asset, key, property, writable)
    elif isinstance(property, cheekpouches.AssetColor):
        return AssetColorEditor(parent, asset, key, property, writable)
    elif isinstance(property, cheekpouches.AssetVec2d):
        return AssetVec2DEditor(parent, asset, key, property, writable)
    elif isinstance(property, cheekpouches.AssetList):
        return AssetListEditor(parent, asset, key, property, writable)
    elif isinstance(property, cheekpouches.AssetObject):
        return AssetObjectEditor(parent, asset, key, property, writable)
    else:
        return wx.StaticText(parent, label = "No Editor for Property "+key+" of type "+_friendly_class_name(type(property)))

########################################################################
class SmartChildFocusScrollPanel(wx.lib.scrolledpanel.ScrolledPanel):
    # at time of writing, ScrolledPanel will scroll to show the top
    # of the direct child with focus - meaning a big child panel will
    # cause the scrolledpanel to scroll away from the child's childs
    # that actually got focus...
    def OnChildFocus(self, evt):
        evt.Skip()
        #focus_child = wx.Window.FindFocus()

########################################################################
class AssetEditor(wx.Frame):
    """ A window that lets you edit the various properties of an asset
        """
    _AssetWindows = {}
    def _Edit(asset, parent = None):
        if not AssetEditor._AssetWindows.has_key(asset):
            window = AssetEditor(parent, asset)
            AssetEditor._AssetWindows[asset] = window
            window.Show()
        else:
            AssetEditor._AssetWindows[asset].Raise()
        return AssetEditor._AssetWindows[asset]
    Edit = staticmethod(_Edit)
    
    def __init__(self, parent, asset):
        path = asset.asset_metadata['path']
        title = 'Editing '+path
        try:
            title = title+' from '+path.root
        except:
            pass
        title = title + ' : ' + _friendly_class_name(type(asset))
        wx.Frame.__init__(self, parent, -1, title,
                          style=wx.DEFAULT_FRAME_STYLE)
        self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)

        self.asset = asset
        self._properties = asset.asset_get_properties()

        self.panel = SmartChildFocusScrollPanel(self, -1, style = wx.TAB_TRAVERSAL|wx.NO_BORDER)
        
        verticalbox = wx.BoxSizer(wx.VERTICAL)

        writable = self.asset.asset_is_writable()
        sorted_keys = self._properties.keys()
        sorted_keys.sort()
        for key in sorted_keys:
            property = self._properties[key]
            editor_panel = _create_property_editor(self.panel, self.asset, key, property, writable)
            item = verticalbox.Add(editor_panel, 0, wx.EXPAND)

        as_image = asset.asset_editor_image()
        if (as_image):
            image_panel = AssetImagePanel(self.panel, asset, as_image)
            item = verticalbox.Add(image_panel, 0, wx.EXPAND)
            
        self.panel.SetSizerAndFit(verticalbox)
        self.panel.SetupScrolling()
        self.panel.SetClientSize(verticalbox.GetSize())
        self.SetClientSize(self.panel.GetSize())
        
            
    #---------------------------------------------
    def OnCloseWindow(self, event):
        self.Destroy()
        self.asset.asset_save()
        del AssetEditor._AssetWindows[self.asset]


########################################################################
_ValidTextEventType = wx.NewEventType() 
EVT_VALID_TEXT = wx.PyEventBinder(_ValidTextEventType, 1) 

class _ValidTextEvent(wx.PyCommandEvent): 
    eventType = _ValidTextEventType 
    def __init__(self, windowID): 
        wx.PyCommandEvent.__init__(self, self.eventType, windowID) 
 
    def Clone( self ): 
        self.__class__( self.GetId() ) 
        
########################################################################
class NumEditCtrl(wx.TextCtrl):
    def __init__(self, parent, min_val, max_val, allow_decimal = False, step = 1, max_width_digits = 15):
        wx.TextCtrl.__init__(self, parent)
        self.min = min_val
        self.max = max_val
        self.max_width_digits = max_width_digits
        self.step = step
        self.allow_decimal = allow_decimal
        self.DetermineMinSize()
        self.Bind(wx.EVT_CHAR, self.FilterChar)
        self.Bind(wx.EVT_TEXT, self.OnEdit)
        self.valid = True
        self.num_evt_text_to_ignore = 0
        
    def DetermineMinSize(self):
        sample_str = " "
        if (self.min < 0):
            sample_str = "-"
        num_digits = 3
        if (self.min != 0):
            num_digits = max(num_digits, int(math.ceil(math.log(abs(self.min), 10))))
        if (self.max != 0):
            num_digits = max(num_digits, int(math.ceil(math.log(abs(self.max), 10))))
        num_digits = min(num_digits, self.max_width_digits)
        sample_str = sample_str + ("0" * (num_digits + 1))
        if (self.allow_decimal):
            sample_str = sample_str + ".000"
        self.SetClientSize(self.GetTextExtent(sample_str))
        self.SetMinSize(self.GetSize())
       
    def FilterChar(self, event):
        key = event.GetKeyCode()
        if key > 47 and key < 58:    # numeric
            event.Skip()
        elif key == 46 and self.allow_decimal and "." not in self.GetValue():    # decimal
            event.Skip()
        elif key == 45 and self.min < 0 and self.GetInsertionPoint() == 0:    # minus sign
            event.Skip()
        elif key == 8 or key == 127:    # backspace/delete
            event.Skip()
        elif key == wx.WXK_LEFT or key == wx.WXK_RIGHT or key == wx.WXK_HOME or key == wx.WXK_END:    # left/right arrow, home/end
            event.Skip()
        elif self.step > 0:
            if key == wx.WXK_UP:    # up arrow
                self.SetNum(self.GetNum() + self.step, send_valid_event = True)
            elif key == wx.WXK_DOWN:    # down arrow
                self.SetNum(self.GetNum() - self.step, send_valid_event = True)
            elif key == wx.WXK_PAGEUP:    # page up
                self.SetNum(self.GetNum() + self.step*10, send_valid_event = True)
            elif key == wx.WXK_PAGEDOWN:    # page down
                self.SetNum(self.GetNum() - self.step*10, send_valid_event = True)
                              
    def SetNum(self, value, send_valid_event = False):
        if not send_valid_event:
            self.num_evt_text_to_ignore = self.num_evt_text_to_ignore + 1
        self.SetValue(str(max(min(value, self.max), self.min)).strip())
        
    def GetNum(self):
        if (self.allow_decimal):
            try:
                return float(self.GetValue())
            except:
                return .0
        else:
            try:
                return int(self.GetValue())
            except:
                return 0

    def OnEdit(self, event):
        value = self.GetNum()
        valid = (value >= self.min and value <= self.max)
        if valid:     
            self.SetForegroundColour(wx.BLACK)
            if self.num_evt_text_to_ignore == 0:
                event = _ValidTextEvent( self.GetId() ) 
                self.GetEventHandler().AddPendingEvent( event ) 
        else:
            self.SetForegroundColour(wx.RED)
        if valid != self.valid:
            self.Refresh()
            self.valid = valid
        if self.num_evt_text_to_ignore > 0:
            self.num_evt_text_to_ignore = self.num_evt_text_to_ignore - 1
        
########################################################################
class AssetPropertyPanel(wx.Panel):
    def __init__(self, parent, asset, key, property, writable):
        wx.Panel.__init__(self, parent, -1, style = wx.TAB_TRAVERSAL|wx.SUNKEN_BORDER)
        self.asset = asset
        self.key = key
        self.property = property
        self.writable = writable

        self.hbox = wx.BoxSizer(wx.HORIZONTAL)
        self.title = wx.StaticText(self, label = key + ":")
        self.hbox.Add(self.title, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)

        self.CreateEditorControls()

        self.clear_button = wx.Button(self, -1, "Set to default")
        self.Bind(wx.EVT_BUTTON, self.SetToDefault, self.clear_button)
        self.clear_button.Enable(not property.asset_has_default(asset, key) and self.writable)
        self.hbox.Add(self.clear_button, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)
        self.SetSizer(self.hbox)        
        self.hbox.SetSizeHints(self)
        if (self.property.description != None):
            self.SetToolTip(wx.ToolTip(self.property.description))
            
        self.SetEditorToCurrent()

    def UpdateTitle(self, key):
        self.title.SetLabel(key + ":")
        
    def SetToDefault(self, event):
        delattr(self.asset, self.key)
        self.clear_button.Enable(False)
        self.SetEditorToCurrent()

########################################################################
class AssetNumberEditor(AssetPropertyPanel):
    def __init__(self, parent, asset, key, property, writable, allow_decimal = False):
        self.allow_decimal = allow_decimal
        AssetPropertyPanel.__init__(self, parent, asset, key, property, writable)
        
    def CreateEditorControls(self):
        self.edit = NumEditCtrl(self, self.property.min, self.property.max, allow_decimal = self.allow_decimal, step = self.property.step)
        self.Bind(EVT_VALID_TEXT, self.OnEdit, self.edit)
        self.edit.Enable(self.writable)
        self.hbox.Add(self.edit, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)
        if (self.property.units != ""):
            units = wx.StaticText(self, label = self.property.units)
            self.hbox.Add(units, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)
        
    def OnEdit(self, event):
        self.asset.asset_set_property(self.key, self.edit.GetNum())
        self.clear_button.Enable(True)
    
    def SetEditorToCurrent(self):
        value = getattr(self.asset, self.key)
        self.edit.SetNum(value)

########################################################################
class AssetStringEditor(AssetPropertyPanel):
    def CreateEditorControls(self):
        self.edit = wx.TextCtrl(self)
        self.Bind(wx.EVT_TEXT, self.OnEdit, self.edit)
        self.edit.Enable(self.writable)
        self.hbox.Add(self.edit, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)
        self.ignore_evt_text_count = 0
        
    def OnEdit(self, event):
        if (self.ignore_evt_text_count > 0):
            self.ignore_evt_text_count -= 1
        else:
            value = self.edit.GetValue()
            self.asset.asset_set_property(self.key, value)
            self.clear_button.Enable(True)
    
    def SetEditorToCurrent(self):
        value = getattr(self.asset, self.key)
        self.ignore_evt_text_count += 1
        self.edit.SetValue(value)

########################################################################
class AssetEnumEditor(AssetPropertyPanel):
    def CreateEditorControls(self):
        self.dropdown = wx.Choice(self, -1, choices = self.property.enum)
        self.Bind(wx.EVT_CHOICE, self.OnChoice, self.dropdown)
        self.dropdown.Enable(self.writable)
        self.hbox.Add(self.dropdown, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)
        
    def OnChoice(self, event):
        value = event.GetString()
        self.asset.asset_set_property(self.key, value)
        self.clear_button.Enable(True)
    
    def SetEditorToCurrent(self):
        value = getattr(self.asset, self.key)
        index = self.dropdown.FindString(value)
        self.dropdown.Select(index)

########################################################################
class AssetFlagEditor(AssetPropertyPanel):
    def CreateEditorControls(self):
        self.check = wx.CheckBox(self, -1)
        self.Bind(wx.EVT_CHECKBOX, self.OnCheck, self.check)
        self.hbox.Add(self.check, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)
        
    def OnCheck(self, event):
        value = event.IsChecked()
        self.asset.asset_set_property(self.key, value)
        self.clear_button.Enable(True)
    
    def SetEditorToCurrent(self):
        value = getattr(self.asset, self.key)
        self.check.SetValue(value)

########################################################################
class AssetRefEditor(AssetPropertyPanel):
    def CreateEditorControls(self):
        asset_ids = cheekpouches.AvailableAssetIDs(self.property.type)
        self.dropdown = wx.Choice(self, -1, choices = asset_ids)
        self.Bind(wx.EVT_CHOICE, self.OnChoice, self.dropdown)
        self.dropdown.Enable(self.writable)
        self.hbox.Add(self.dropdown, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)

        self.edit_button = wx.Button(self, -1, "Edit")
        self.Bind(wx.EVT_BUTTON, self.EditSelected, self.edit_button)
        self.hbox.Add(self.edit_button, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)
        
    def OnChoice(self, event):
        class_name = event.GetString()
        self.asset.asset_set_property(self.key, class_name)
        self.clear_button.Enable(True)
    
    def EditSelected(self, event):
        parent = self.GetParent()
        while parent != None and parent.GetParent() != None:
            parent = parent.GetParent()
        if (isinstance(parent, AssetEditor)):
            # Don't want asset editors to have parent/child relationships
            parent = None
        AssetEditor.Edit(getattr(self.asset, self.key), parent)
        
    def SetEditorToCurrent(self):
        id = self.GetAssetID()
        index = self.dropdown.FindString(id)
        self.dropdown.Select(index)
    
    def GetAssetID(self):
        asset = self.asset
        while isinstance(asset, ObjectAssetBridge):
            asset = asset.get_asset()
        return self.property.get_assetid(asset)

########################################################################
class AssetVec2DEditor(AssetPropertyPanel):
    def CreateEditorControls(self):
        title_text = wx.StaticText(self, label = "x:")
        self.hbox.Add(title_text, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)
        self.x = NumEditCtrl(self, -1e300, 1e300, allow_decimal = True, max_width_digits = 6)
        self.x.Enable(self.writable)
        self.Bind(EVT_VALID_TEXT, self.OnEdit, self.x)
        self.hbox.Add(self.x, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)
        title_text = wx.StaticText(self, label = "y:")
        self.hbox.Add(title_text, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)
        self.y = NumEditCtrl(self, -1e300, 1e300, allow_decimal = True, max_width_digits = 6)
        self.y.Enable(self.writable)
        self.Bind(EVT_VALID_TEXT, self.OnEdit, self.y)
        self.hbox.Add(self.y, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)
            
    def AddEditBox(self):
        return edit
        
    def OnEdit(self, event):
        value = vec2d(self.x.GetNum(), self.y.GetNum())
        self.asset.asset_set_property(self.key, value)
        self.clear_button.Enable(not self.property.asset_has_default(self.asset, self.key))
    
    def SetEditorToCurrent(self):
        value = getattr(self.asset, self.key)
        self.x.SetNum(value.x)
        self.y.SetNum(value.y)

########################################################################
class AssetColorEditor(AssetPropertyPanel):
    def CreateEditorControls(self):
        self.include_alpha = len(getattr(self.asset, self.key)) >= 4
        if self.include_alpha:
            title = "rgba"
        else:
            title = "rgb"
        title_text = wx.StaticText(self, label = title+":")
        self.hbox.Add(title_text, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)
        self.red = self.AddEditBox()
        self.green = self.AddEditBox()
        self.blue = self.AddEditBox()
        if self.include_alpha:
            self.alpha = self.AddEditBox()
            
    def AddEditBox(self):
        edit = NumEditCtrl(self, .0, 1.0, allow_decimal = True, step = .01)
        edit.Enable(self.writable)
        self.Bind(EVT_VALID_TEXT, self.OnEdit, edit)
        self.hbox.Add(edit, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)
        return edit
        
    def OnEdit(self, event):
        if self.include_alpha:
            value = [self.red.GetNum(), self.green.GetNum(), self.blue.GetNum(), self.alpha.GetNum()]
        else:
            value = [self.red.GetNum(), self.green.GetNum(), self.blue.GetNum()]
        self.asset.asset_set_property(self.key, value)
        self.clear_button.Enable(not self.property.asset_has_default(self.asset, self.key))
    
    def SetEditorToCurrent(self):
        value = getattr(self.asset, self.key)
        self.red.SetNum(value[0])
        self.green.SetNum(value[1])
        self.blue.SetNum(value[2])
        if self.include_alpha:
            self.alpha.SetNum(value[3])

########################################################################
class ListAssetBridge(object):
    def __init__(self, asset, key):
        self.asset = asset
        self.key = key
        
    def __getattr__(self, key):
        list = self.get_list()
        return list[int(key)]
    
    def asset_set_property(self, key, value):
        list = self.get_list()
        list[int(key)] = value
        self.asset.asset_set_property(self.key, list)

    def asset_report_property_change(self, key, value):
        self.asset.asset_report_property_change(self.key, self.get_list())
        
    def get_list(self):
        return getattr(self.asset, self.key)
    
########################################################################
class AssetListEditor(wx.Panel):

    def __init__(self, parent, asset, key, property, writable):
        wx.Panel.__init__(self, parent, -1, style = wx.TAB_TRAVERSAL|wx.SUNKEN_BORDER)
        self.asset = asset
        self.key = key
        self.property = property
        self.writable = writable
        self.editors = []

        self.hbox = wx.BoxSizer(wx.HORIZONTAL)
        title = wx.StaticText(self, label = key + ":")
        self.hbox.Add(title, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)

        self.list_box = wx.BoxSizer(wx.VERTICAL)
        self.element_box = wx.BoxSizer(wx.VERTICAL)

        self.list_box.Add(self.element_box, 0, border = 2, flag = wx.ALL)
        
        self.append_button = wx.Button(self, -1, "Append new element")
        self.Bind(wx.EVT_BUTTON, self.AppendElement, self.append_button)
        self.list_box.Add(self.append_button, 0, border = 2, flag = wx.FIXED_MINSIZE | wx.ALL)
        
        self.list_bridge = ListAssetBridge(self.asset, self.key)
        list = getattr(self.asset, self.key)
        for index in xrange(len(list)):
            element_panel = self.AddElementControls(index)
        
        self.hbox.Add(self.list_box, 0, border = 5, flag = wx.ALL)
        self.SetSizer(self.hbox)        
        self.hbox.SetSizeHints(self)
        if (self.property.description != None):
            self.SetToolTip(wx.ToolTip(self.property.description))

    def AddElementControls(self, index):
        editor = _create_property_editor(self, self.list_bridge, str(index), self.property.descriptor, self.writable)
        editor_box = wx.BoxSizer(wx.HORIZONTAL)
        editor_box.Add(editor, 0, border = 0, flag = wx.FIXED_MINSIZE | wx.ALL)
        delete_button = wx.Button(self, index, "del")
        self.Bind(wx.EVT_BUTTON, self.DeleteElement, delete_button)
        editor_box.Add(delete_button, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)
        self.element_box.Add(editor_box, 0, border = 0, flag = wx.ALL )
        self.editors.append((editor_box, editor, delete_button))
    
    def AppendElement(self, event):
        list = getattr(self.asset, self.key)
        list.append(self.property.descriptor.get_new())
        self.AddElementControls(len(list) - 1)
        self.hbox.SetSizeHints(self)
        self.UpdateParentSize()

    def DeleteElement(self, event):
        list = getattr(self.asset, self.key)
        del list[event.Id]
        self.element_box.Remove(self.editors[event.Id][0])
        self.editors[event.Id][2].Destroy()
        self.editors[event.Id][1].Destroy()
        del self.editors[event.Id]
        
        new_index = 0
        for editor_box, editor, delete_button in self.editors:
            editor.UpdateTitle(str(new_index))
            delete_button.SetId(new_index)
            new_index += 1
        self.hbox.SetSizeHints(self)
        self.UpdateParentSize()
        
    def UpdateParentSize(self):
        parent = self.GetParent()
        while parent != None and not isinstance(parent, AssetEditor):
            parent = parent.GetParent()
        best_size = parent.GetBestSize()
        current_size = parent.GetSize()
        new_size = (max(best_size[0], current_size[0]), max(best_size[1], current_size[1])+1)
        parent.SetSize(new_size)
        parent.Fit()        
            
########################################################################
class ObjectAssetBridge(object):
    def __init__(self, parent_asset, key):
        self.parent_asset = parent_asset
        self.key = key
        
    def __getattr__(self, key):
        return getattr(getattr(self.parent_asset, self.key), key)
    
    def asset_set_property(self, key, value):
        asset = getattr(self.parent_asset, self.key)
        asset.asset_set_property(key, value)
        self.parent_asset.asset_report_property_change(self.key, asset)

    def asset_report_property_change(self, key, value):
        asset = getattr(self.parent_asset, self.key)
        asset.asset_report_property_change(key, value)
        self.parent_asset.asset_report_property_change(self.key, asset)
        
    def get_asset(self):
        return getattr(self.parent_asset, self.key)
    
########################################################################
class AssetObjectEditor(wx.Panel):

    def __init__(self, parent, asset, key, property, writable):
        wx.Panel.__init__(self, parent, -1, style = wx.TAB_TRAVERSAL|wx.SUNKEN_BORDER)
        self.asset = asset
        self.key = key
        self.property = property
        self.writable = writable

        self.hbox = wx.BoxSizer(wx.HORIZONTAL)
        title = wx.StaticText(self, label = key + ":")
        self.hbox.Add(title, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)

        self.element_box = wx.BoxSizer(wx.VERTICAL)

        object = ObjectAssetBridge(self.asset, self.key)
        self._properties = object.asset_get_properties()
        sorted_keys = self._properties.keys()
        sorted_keys.sort()
        for object_key in sorted_keys:
            property = self._properties[object_key]
            editor_panel = _create_property_editor(self, object, object_key, property, self.writable)
            item = self.element_box.Add(editor_panel, 0, wx.EXPAND)
        
        self.hbox.Add(self.element_box, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)
        self.SetSizer(self.hbox)        
        self.hbox.SetSizeHints(self)
        if (self.property.description != None):
            self.SetToolTip(wx.ToolTip(self.property.description))

    def UpdateParentSize(self):
        parent = self.GetParent()
        parent = parent.GetParent()
        parent.Fit()        
        
########################################################################
class AssetImagePanel(wx.Panel):
    def __init__(self, parent, asset, image):
        wx.Panel.__init__(self, parent, -1, style = wx.SUNKEN_BORDER)
        self.asset = asset
        self.asset.asset_monitor_changes(self)
        self.SetImage(image)
        self.Bind(wx.EVT_PAINT, self.OnPaint)

    def SetImage(self, image):
        image_string = pygame.image.tostring(image, "RGB")
        imgWx = wx.EmptyImage(image.get_width(), image.get_height())
        self.SetClientSize((image.get_width(), image.get_height()))
        self.SetMinSize(self.GetSize())
        imgWx.SetData(image_string)
        self.bitmap = imgWx.ConvertToBitmap()
    
    def OnPaint(self, evt):
        dc = wx.PaintDC(self)
        dc.SetBackground(wx.Brush("WHITE"))
        dc.Clear()
        dc.DrawBitmap(self.bitmap, 0, 0, True)

    def asset_changed(self, asset, key, value):
        print "getting new image"
        self.SetImage(self.asset.asset_editor_image())
        self.Refresh()
        
########################################################################
## Quick Testing                                                       ##
########################################################################
if __name__ == "__main__":

    from motherhamster import enums
    ########################################################################
    class EmbeddedClass(cheekpouches.AssetBase):
        ReflectedFloat = cheekpouches.AssetFloat(1.0, "x", 0, 1)
        ReflectedString = cheekpouches.AssetString("embedded")

        def asset_is_writable(self):
            return True
        
    class SimpleClass(cheekpouches.AssetBase):
        def __init__(self, writable = True):
            cheekpouches.AssetBase.__init__(self)
            self.writable = writable
            
        def asset_is_writable(self):
            return self.writable

        def asset_editor_image(self):
            asset_image = pygame.Surface((100,150), 32)
            asset_image.fill(self.ReflectedColorNoAlpha)
            circle_pos = (int(self.ReflectedFloat*100), self.ReflectedIntMinMax100)
            radius = self.ReflectedIntOddRange
            pygame.draw.circle(asset_image, self.ReflectedColor, circle_pos, radius)
            return asset_image
        
        MyEnum = enums.Enum("Option1", "Option2", "Option3")
        
        ReflectedFlag = cheekpouches.AssetFlag(False)
        ReflectedFloat = cheekpouches.AssetFloat(1.0, "x", 0, 1)
        ReflectedIntOddRange = cheekpouches.AssetInteger(50, "units", 20, 500)
        ReflectedIntMinMax100 = cheekpouches.AssetInteger(0, "units", -100, 100)
        ReflectedString = cheekpouches.AssetString("default")
        ReflectedChoice = cheekpouches.AssetEnum(MyEnum)
        ReflectedVector = cheekpouches.AssetVec2d()
        ReflectedColor = cheekpouches.AssetColor([1.0,.5,0,1.0])
        ReflectedColorNoAlpha = cheekpouches.AssetColor([0,0,0], "this color has no alpha")
        ReflectedColorList = cheekpouches.AssetList(cheekpouches.AssetVec2d(), "testing lists")
        ReflectedObject = cheekpouches.AssetObject(EmbeddedClass)
        Reference = None
            
    # silly thing doesn't let me reference the class type in the definition of the class..
    EmbeddedClass.Reference = cheekpouches.AssetRef(SimpleClass, "SimpleReference", "Test of Reference Property within an object property")
    SimpleClass.Reference = cheekpouches.AssetRef(SimpleClass, "SimpleReference", "Test of Reference Property, is a reference to an asset in the pouch this list comes from")
    
    ########################################################################
    class TestCheekPouch(cheekpouches.GenericAssetPouch):

        def __init__(self):
            cheekpouches.GenericAssetPouch.__init__(self, SimpleClass)
            self._ValidAssetList = {"FirstSimple":SimpleClass(), "SimpleReference":SimpleClass(), "NotWritable":SimpleClass(False)}
            
        def _load(self, assetid):
            if (self._ValidAssetList.has_key(assetid)):
                return self._ValidAssetList[assetid]
            return None

        def _save(self, assetid, obj):
            print "Was asked to save "+assetid
            
        def AddAsset(self, ID, value):
            self._ValidAssetList[ID] = value
            
        def DelAsset(self, ID):
            del self._loaded_assets[ID]
            del self._ValidAssetList[ID]
        
        def AvailableAssetIDs(self):
            return self._ValidAssetList.keys()
            

    ########################################################################
    RegisteredPouch = TestCheekPouch()
    cheekpouches.RegisterPouch(RegisteredPouch)

    ########################################################################
    class _TestApp(wx.App):
        def __init__(self):
            self.Window = None
            wx.App.__init__(self, redirect=False, useBestVisual=True)
            
        def OnInit(self):
            self.Window = AssetEditor.Edit(cheekpouches.Disgorge(SimpleClass, "FirstSimple"))
            self.Window.Show()
            return True
        
    app = _TestApp()
    app.MainLoop()

    
########################################################################
        