"""
Virtual File System stuff
(C) Copyright 2004 Hamster Republic Productions
"""

import os.path
import sys

########################################################################
class FoodDishDirectory(object):
    """Simple class for a file/directory abstraction for finding assets"""
    def __init__(self, relative_path):
        self._root_path = os.path.join(os.path.dirname(sys.argv[0]), relative_path)
        self.name = relative_path
    
    def FindSubdirectoryPaths(self):
        dirlist = []
        if os.path.exists(self._root_path):
            self._RecursiveDirectorySearch(dirlist, "")
        return dirlist
        
    def _RecursiveDirectorySearch(self, list, path):
        full_path = os.path.join(self._root_path, path)
        dirlist = os.listdir(full_path)
        for file_dir in dirlist:
            full_file_dir_path = os.path.join(full_path, file_dir)
            if os.path.isdir(full_file_dir_path):
                stem, ext = os.path.splitext(file_dir)
                # directories with extensions can be asset types as well
                # and we don't want to dig into directories we don't understand
                if ext == "":
                    localdirpath = os.path.join(path, file_dir)
                    list.append(localdirpath.replace("\\", "/"))
                    self._RecursiveDirectorySearch(list, localdirpath)

    def GetAssetPath(self, directory, asset_id, extension):
        filename = asset_id + extension.lower()
        return AssetPath(os.path.join(directory, filename), self.name)
    
    def FindFilesByExtension(self, extension_list):
        """given a list of extensions (lowercase, including the .),
           this routine will return a list of all the valid asset_path's"""
        filelist = []
        if os.path.exists(self._root_path):
            self._RecursiveFileExtensionSearch(filelist, "", extension_list)
        return filelist
    
    def _RecursiveFileExtensionSearch(self, list, path, extension_list):
        full_path = os.path.join(self._root_path, path)
        dirlist = os.listdir(full_path)
        for file_dir in dirlist:
            file_dirpath = os.path.join(full_path, file_dir)
            if os.path.isdir(file_dirpath):
                stem, ext = os.path.splitext(file_dir)
                # directories with extensions can be asset types as well
                # and we don't want to dig into directories we don't understand
                if ext == "":
                    self._RecursiveFileExtensionSearch(list, os.path.join(path, file_dir), extension_list)
                elif ext in extension_list:
                    list.append(AssetPath(os.path.join(path, file_dir), self.name))
            elif os.path.isfile(file_dirpath):
                ext = os.path.splitext(file_dir)[1].lower()
                temp = ext in extension_list
                if ext in extension_list:
                    list.append(AssetPath(os.path.join(path, file_dir), self.name))
    
    def GetRootPath(self):
        return self._root_path
    
    def open(self, asset_path, open_flags):
        full_path = self._get_native_filename(asset_path)
        return open(full_path, open_flags)        

    def is_writable(self, asset_path):
        full_path = self._get_native_filename(asset_path)
        while full_path != "" and not os.path.exists(full_path):
            full_path, new_file = os.path.split(full_path)
        return os.access(full_path, os.W_OK)
            
    def _get_native_filename(self, asset_path):
        """returns a string that is a path to a file on the local file system
           that can be opened to get the asset contents"""        
        full_path = os.path.join(self._root_path, asset_path.replace("/", os.path.sep))
        if (hasattr(asset_path, "extension")):
            full_path = full_path + asset_path.extension
        return full_path
    
########################################################################
class AssetPath(str):
    def __new__(cls, path, root=None):
        noextension, extension = os.path.splitext(path)
        noextension = noextension.replace("\\", "/")
        s = str.__new__(cls, noextension)
        s._ext = extension
        s._root = root
        return s
    
    def _getroot(self):
        return self._root
        
    def _getext(self):
        return self._ext
        
    def _getfilepath(self):
        return self + self._ext
    
    def _getname(self):
        return os.path.split(self)[1]
    
    def _getpath(self):
        return os.path.split(self)[0]
    
    path = property(_getpath)
    assetid = property(_getname)
    filepath = property(_getfilepath)
    extension = property(_getext)
    root = property(_getroot)


########################################################################
## Unit Testing                                                       ##
########################################################################
if __name__ == "__main__":
    import unittest
    import shutil

    ########################################################################
    class UnitTestFD(unittest.TestCase):
        def setUp(self):
            pass
        
        def testAssetPaths(self):
            path_string = 'images\\thing.jpg'
            ap1 = AssetPath(path_string, 'data')
            self.assert_(ap1.path == 'images')
            self.assert_(ap1.assetid == 'thing')
            self.assert_(ap1.filepath == 'images/thing.jpg')
            self.assert_(ap1.extension == '.jpg')
            self.assert_(ap1.root == 'data')
            self.assert_(ap1 == 'images/thing')
    
        def testFoodDish(self):
            try:
                temp_dir_string = 'unittest_fooddish_temp_unique_name'
                fooddir = FoodDishDirectory(temp_dir_string)
                full_path = fooddir.GetRootPath()
                if not os.path.isdir(full_path):
                    os.mkdir(full_path)
                images_path = os.path.join(full_path, 'images')
                if not os.path.isdir(images_path):
                    os.mkdir(images_path)
                images_subpath = os.path.join(images_path, 'Misc')
                if not os.path.isdir(images_subpath):
                    os.mkdir(images_subpath)
                ignored_path = os.path.join(full_path, 'dir.ignore')
                if not os.path.isdir(ignored_path):
                    os.mkdir(ignored_path)
                # testing that extension is preserved, but still matched when case is different
                image_file = os.path.join(images_path, 'thing.JpG')
                if not os.path.isfile(image_file):
                    file = open(image_file, 'w')
                    file.write("dummy testing file")
                    file.close()

                ignored_image = os.path.join(ignored_path, 'ignored.Jpg')
                if not os.path.isfile(ignored_image):
                    file = open(ignored_image, 'w')
                    file.write("dummy testing file")
                    file.close()
                    
                list = fooddir.FindFilesByExtension(['.jpg'])
                self.assert_(len(list) == 1)
                if (len(list) == 1):                
                    ap1 = list[0]
                    self.assert_(ap1.path == 'images')
                    self.assert_(ap1.assetid == 'thing')
                    self.assert_(ap1.filepath == 'images/thing.JpG')
                    self.assert_(ap1.extension == '.JpG')
                    self.assert_(ap1.root == temp_dir_string)
                    self.assert_(ap1 == 'images/thing')
                    file = fooddir.open(ap1.filepath, "r")
                    self.assert_(file.read() == "dummy testing file")
                    self.assertEqual(fooddir._get_native_filename(ap1), os.path.join(images_path, "thing.JpG"))
                    file.close()
                    
                dirlist = fooddir.FindSubdirectoryPaths()
                self.assert_(len(dirlist) == 2)
                if (len(dirlist) == 2):                
                    dir1 = dirlist[0]
                    self.assert_(dir1 == "images")
                    dir2 = dirlist[1]
                    self.assert_(dir2 == "images/Misc")
            finally:
                shutil.rmtree(full_path, True)
                self.assert_(not os.path.isdir(full_path))
    
    ########################################################################

    unittest.main()
    