"""
Image Loader
(C) Copyright 2004 Hamster Republic Productions
"""

########################################################################
from pygame import Surface
import pygame.imageext
import pygame.image
from motherhamster.cheekpouches import *
from motherhamster.mathobj import vec2d
from motherhamster.enums import Enum

# CHECK: will eventually replace directory stuff with some other layer?
import os.path
import sys
import pickle
import weakref
import types
import math

########################################################################
_EstimatedSurfaceMemory = 0

def EstimateSurfaceMemory():
    global _EstimatedSurfaceMemory
    return _EstimatedSurfaceMemory

def CountSurfaceMemory(surface):
    global _EstimatedSurfaceMemory
    size = surface.get_width()*surface.get_height()*surface.get_bytesize()
    _EstimatedSurfaceMemory += size
    return size

def SubtractSurfaceMemory(size):
    global _EstimatedSurfaceMemory
    if _EstimatedSurfaceMemory != None:
        _EstimatedSurfaceMemory -= size

########################################################################
class ImageAsset(AssetBase):
    """a python object representation of an image asset
       it represents both the raw image data and the metadata
       it can produce pygame surfaces for use in rendering"""
    
    x_origins = Enum("left", "center", "right")
    y_origins = Enum("top", "center", "bottom")
    blend_modes = Enum("alpha_blend", "additive", "lighting")
    
    hotspotx_type = AssetEnum(x_origins, x_origins.center)
    hotspoty_type = AssetEnum(y_origins, y_origins.center)
    hotspot_offset = AssetVec2d()
    blend_mode = AssetEnum(blend_modes, blend_modes.alpha_blend)
    joints = AssetList(AssetVec2d())
    
    def __init__(self):
        AssetBase.__init__(self)

    def _set_file(self, file):
        if hasattr(self, "_memory_estimate"):
            global SubtractSurfaceMemory
            SubtractSurfaceMemory(self._memory_estimate)
        self.__rawimage = pygame.image.load(file)
        global CountSurfaceMemory
        self._memory_estimate = CountSurfaceMemory(self.__rawimage)
        
    def __del__(self):
        global SubtractSurfaceMemory
        if (SubtractSurfaceMemory != None):
            SubtractSurfaceMemory(self._memory_estimate)
    
    def _get_hotspot(self):
        calculated_hotspot = self.calculate_hotspot()
        self.__dict__["hotspot"] = calculated_hotspot
        return calculated_hotspot
    hotspot = property(_get_hotspot, doc="gets the hotspot for the image")
    
    def _get_size(self):
        return vec2d(self.__rawimage.get_width(), self.__rawimage.get_height())        
    size = property(_get_size, doc="gets the default pixel size of the image for the asset")

    def _get_width(self):
        return self.__rawimage.get_width()
    width = property(_get_width, doc="gets the default pixel width of the image for the asset")
    
    def _get_height(self):
        return self.__rawimage.get_height()
    height = property(_get_height, doc="gets the default pixel height of the image for the asset")

    def get_hotspot_base(self):
        if (self.hotspotx_type == "center"):
            x = self.__rawimage.get_width()/2.0
        elif (self.hotspotx_type == "right"):
            x = self.__rawimage.get_width()
        else:
            x = 0    
        if (self.hotspoty_type == "center"):
            y = self.__rawimage.get_height()/2.0
        elif (self.hotspoty_type == "top"):
            y = self.__rawimage.get_height()
        else:
            y = 0
        return vec2d(x, y)
        
    def calculate_hotspot(self):
        return self.get_hotspot_base() + self.hotspot_offset

    def get_joint_offset(self, joint_name):
        try:
            return self.joints[joint_name]
        except:
            return vec2d(0,0)

    def compute_joint_pos(self, joint_name, pos, angle=None, scale=None):
        try:
            joint_pos = self.joints[joint_name]
        except:
            return pos
        if scale != None:
            joint_pos = joint_pos*scale
        if angle != None:
            joint_pos = joint_pos.rotated(angle)
        joint_pos += pos
        return joint_pos
    
    def __get_surface(self):
        return self.__rawimage
    surface = property(__get_surface, doc="gets the pygame surface for this image")

    def asset_editor_image(self):
        asset_image = self.__rawimage.convert(32)
        inverted_hotspot = vec2d(int(self.hotspot.x), int(self.height - self.hotspot.y))
        pygame.draw.aaline(asset_image, (0,0,255, 96), inverted_hotspot, inverted_hotspot, 1)
        pygame.draw.circle(asset_image, (255,0,255), inverted_hotspot, 4, 1)
        pygame.draw.line(asset_image, (255,0,255), inverted_hotspot - (5,0), inverted_hotspot + (5,0))
        pygame.draw.line(asset_image, (255,0,255), inverted_hotspot - (0,5), inverted_hotspot + (0,5))
        for joint_offset in self.joints:
            joint_pos = inverted_hotspot + (joint_offset.x, -joint_offset.y)
            pygame.draw.line(asset_image, (0,255,255), joint_pos + (-2,-2), joint_pos + (2,2), 1)
            pygame.draw.line(asset_image, (0,255,255), joint_pos + (2,-2), joint_pos + (-2,2), 1)
        return asset_image

    def asset_set_property(self, key, value):
        AssetBase.asset_set_property(self, key, value)
        if "hotspot" in self.__dict__:
            del self.__dict__["hotspot"]
        if hasattr(self, "image_on_change"):
            self.image_on_change(self)
              
########################################################################
class _ImagePouch(GenericAssetPouch):
    """the base class for asset loaders"""

    def __init__(self):
        GenericAssetPouch.__init__(self, ImageAsset)
        self._asset_paths = None

    def AssetExtensions(self):
        return [".png", ".jpg"]

    def _load(self, asset_path):
        """loads an asset using pickle for the data part and also loads some image part..."""
        path = self.GetAssetPathForID(asset_path)
        if (path):
            dish = GetFoodDish(path.root)
            metadata_filename = path + '.image'
            try:
                metadata_file = dish.open(metadata_filename, 'rb')
            except:
                metadata_file = None
            if (metadata_file):
                # CHECK: this surely won't work right...
                asset = pickle.load(metadata_file)
            else:
                asset = ImageAsset()
            asset._set_file(dish.open(path.filepath, "rb"))
            return asset
        return None

    def _save(self, asset_path, obj):
        """saves an asset using pickle (just the data part)"""
        path = self.GetAssetPathForID(asset_path)
        if (path):
            dish = GetFoodDish(path.root)
            metadata_filename = path + '.image'
            metadata_file = dish.open(metadata_filename, 'wb')
            pickle.dump(obj, metadata_file)

########################################################################

pouch = _ImagePouch()
RegisterPouch(pouch)

if __name__ == "__main__":
    #TODO: imagepouch unit tests
    pass