"""
filters and effects for pygame surfaces
(C) Copyright 2006 Hamster Republic Productions
"""

########################################################################
import pygame.surface
import pygame.image
import array
from motherhamster.cheekpouches import *
from motherhamster.mathobj import vec2d

########################################################################
class SurfaceFilterABC(AssetBase):
    """an abstract base class for filters"""
    def get_expansion(self):
        return (0,0,0,0)
    
########################################################################
class TexturizeFilter(SurfaceFilterABC):
    """a filter that remaps the opaque pixels to a texture"""
    def __init__(self):
        pass

########################################################################
class AlphaBlurBehindFilter(SurfaceFilterABC):
    """a filter that puts a colored blur of the alpha channel behind the surface, like outer glow or drop shadow"""
    radius = AssetInteger(2, "pixels", 0, 16, "the amount to blur")
    offset = AssetVec2d(vec2d(2, 2), "the offset for the blur")
    color = AssetColor([0,0,0,192], "the color of the blur")

    def process_surface(self, surface):
        size = surface.get_size()
        blur_buffer, blur_size = GetAlphaBlur(surface, self.radius)
        blur_surface = CreateSurfaceFromAlpha(blur_buffer, blur_size, self.color)
        hotspot_offset = vec2d(max(0, self.radius - self.offset.x), max(0, self.radius - self.offset.y))
        blur_offset = offset - (self.radius, self.radius)
        final_size = (size[0] + max(self.radius*2, abs(self.offset.x) + self.radius), size[1] + max(self.radius*2, abs(self.offset.y) + self.radius))
        final_surface = pygame.surface.Surface(final_size, pygame.SRCALPHA, 32)
        final_surface.blit(blur_surface, blur_offset)
        final_surface.blit(surface, hotspot_offset)
        return final_surface, hotspot_offset
        
#CHECK: need to implement a filtering object that can contain
# multiple ordered filters in it. They should be editable assets
# When run on a surface, they can affect the size, but need to return
# how the size is affected
########################################################################
class SurfaceFilters(AssetBase):
    #CHECK: need to make this a list of filters, when you can have a descriptor for a list
    filter = AssetObject(SurfaceFilterABC, AlphaBlurBehindFilter())
    max_crop_alpha = AssetInteger(4, "alpha", 0, 255, "the highest alpha value to throw away when cropping")
    
    def __init__(self):
        pass
    
    def process_surface(self, original_surface):
        minRect = FindCropRect(original_surface, self.max_crop_alpha)
        hotspot_offset = vec2d(minRect.left,  minRect.top)
        width = minRect.width
        height = minRect.height
        filtered_surface = pygame.surface.Surface((minRect.width, minRect.height), pygame.SRCALPHA, 32)
        filtered_surface.blit(original_surface,(0,0), minRect)
        if self.filter != None:
            filtered_surface, filter_offset = self.filter.process_surface(filtered_surface)
            hotspot_offset += filter_offset
        return filtered_surface, hotspot_offset
        

########################################################################
def _LineScanBuffer(buffer, offset, step_size, num_steps, line_size, max_lines, max_value):
    line_number = 0
    while line_number < max_lines:
        start_index = line_number*line_size + offset
        for line_index in xrange(0, num_steps*step_size, step_size):
            if buffer[start_index + line_index] > max_value:
                return line_number
        line_number += 1
    return max_lines
            
def FindCropRect(surface, max_alpha = 0):
    img_buf = pygame.image.tostring(surface, "RGBA")
    width, height = surface.get_size()
    x_step = 4
    y_step = width*x_step
    min_x = _LineScanBuffer(img_buf, 3, y_step, height, x_step, width, chr(max_alpha))
    
    y_scan_top_left = min_x*x_step + 3
    min_y = _LineScanBuffer(img_buf, y_scan_top_left, x_step, width - min_x, y_step, height, chr(max_alpha))
    
    x_scan_top_right = min_y*y_step + (width - 1)*x_step + 3
    max_x = (width - 1) - _LineScanBuffer(img_buf, x_scan_top_right, y_step, height - min_y, -x_step, width - min_x, chr(max_alpha))
    
    y_scan_bottom_left = (height - 1)*y_step + min_x*x_step + 3
    max_y = (height - 1) - _LineScanBuffer(img_buf, y_scan_bottom_left, x_step, max_x - min_x + 1, -y_step, height - min_y, chr(max_alpha))
    
    return pygame.Rect(min_x, min_y, max_x - min_x + 1, max_y - min_y + 1)

def _GaussianBlurBuffer1D(radius, num_steps, num_lines, src, src_offset, src_step_size, src_line_step, dest, dest_offset, dest_step_size, dest_line_step):
    depth = radius*2
    div = 2**depth
    rounder = div/2
    for y in xrange(num_lines):
        src_index = src_offset + y*src_line_step
        dest_index = dest_offset + y*dest_line_step
        husk = [0]*depth
        for x in xrange(num_steps):
            new_value = src[src_index]
            src_index += src_step_size
            for i in xrange(depth):
                replace_value = new_value
                new_value = husk[i] + replace_value
                husk[i] = replace_value
            dest[dest_index] = (new_value+rounder)/div
            dest_index += dest_step_size
        for x in xrange(radius*2):
            new_value = 0
            for i in xrange(radius*2):
                replace_value = new_value
                new_value = husk[i] + replace_value
                husk[i] = replace_value
            dest[dest_index] = (new_value + rounder)/div
            dest_index += dest_step_size            
            
def GetAlphaBlur(surface, radius):
    rgba_buffer = array.array("B", pygame.image.tostring(surface, "RGBA"))
    src_width, src_height = surface.get_size()
    blur_width = src_width + radius*2
    blur_height = src_height + radius*2
    blur_buffer = array.array("B", [0]*(blur_width*blur_height))
    _GaussianBlurBuffer1D(radius, src_height, src_width, rgba_buffer, 3, 4*src_width, 4, blur_buffer, radius, blur_width, 1)
    _GaussianBlurBuffer1D(radius, src_width, blur_height, blur_buffer, radius, 1, blur_width, blur_buffer, 0, 1, blur_width)
    return blur_buffer, (blur_width, blur_height)

def CreateSurfaceFromAlpha(alpha_buffer, size, color):    
    color_buffer = array.array("B", color*len(alpha_buffer))
    for i in xrange(len(alpha_buffer)):
        color_buffer[i*4 + 3] = min(color[3]*alpha_buffer[i]/255,255)
    return pygame.image.fromstring(color_buffer.tostring(), size, "RGBA")
    
########################################################################
## Unit Testing                                                       ##
########################################################################
if __name__ == "__main__":
    import pygame.font
    import unittest
    import time

    def noWorkBaselineFunction(arg):
        pass
    
    def timeFunction(func, arg, num_runs = 10000):
        start = time.clock()
        for i in xrange(num_runs):
            func(arg)
        end = time.clock()
        elapsed = end - start
        return (elapsed, num_runs)
    
    def perfTest(func, arg, num_runs = 10000, fps=60, budget=.05):
        baseline = timeFunction(noWorkBaselineFunction, None, num_runs)
        tested = timeFunction(func, arg, num_runs)
        estimated = (tested[0] - baseline[0], num_runs)
        print "%s runs at %.1f loops/s (%.3fms, %.1f%% overhead)" % (str(func), num_runs/estimated[0], 1000.0*tested[0]/num_runs, 100.0*baseline[0]/tested[0])
        print "that's %.2f loops to fill %d fps" % (num_runs/estimated[0]/fps, fps)
        print "or %.2f at a %.1f%% budget at %d fps" % (budget*num_runs/estimated[0]/fps, budget*100.0, fps)

    class UnitTestSurfaceCropping(unittest.TestCase):
        def assertSurfacesIdentical(self, surf_a, surf_b):
            self.assertEqual(surf_a.get_width(), surf_b.get_width())
            self.assertEqual(surf_a.get_height(), surf_b.get_height())
            for y in xrange(surf_a.get_height()):
                for x in xrange(surf_a.get_width()):
                    self.assertEqual((x,y,surf_a.get_at((x,y))), (x,y, surf_b.get_at((x,y))))
            
        def testCroppingPreservesImage(self):
            defaultFont = pygame.font.Font(None, 24)
            period_surface = defaultFont.render(".", True, (255,255,255))

            blit_full = pygame.surface.Surface(period_surface.get_size(), pygame.SRCALPHA, 32)
            blit_full.blit(period_surface, (0,0))
            self.assertSurfacesIdentical(blit_full, period_surface)            

            minRect = FindCropRect(period_surface)
            cropped_surface = pygame.surface.Surface((minRect.width, minRect.height), pygame.SRCALPHA, 32)
            cropped_surface.blit(period_surface, (0,0), minRect)
            self.assertEqual(cropped_surface.get_rect(), FindCropRect(cropped_surface))
            
            blit_partial = pygame.surface.Surface(period_surface.get_size(), pygame.SRCALPHA, 32)
            blit_partial.blit(cropped_surface, (minRect))
            self.assertEqual(minRect, FindCropRect(blit_partial))
            
            self.assertSurfacesIdentical(blit_partial, blit_full)            

        def testCroppingIsTight(self):
            test_surface = pygame.surface.Surface((100,100), pygame.SRCALPHA, 32)
            test_surface.set_at((10,10), (255,255,255,255))
            test_surface.set_at((48,12), (255,255,255,255))
            minRect = FindCropRect(test_surface)     
            self.assertEqual(minRect, pygame.Rect(10, 10, 39, 3))       

        def testBlurNoSizeIsHarmless(self):
            test_surface = pygame.surface.Surface((10,7), pygame.SRCALPHA, 32)
            test_surface.set_at((3,1), (0,0,0,255))
            test_surface.set_at((9,3), (0,0,0,255))
            blur_buffer, blur_size = GetAlphaBlur(test_surface, 0)
            blur_surface = CreateSurfaceFromAlpha(blur_buffer, blur_size, (0,0,0,255))
            self.assertSurfacesIdentical(test_surface, blur_surface)

        def testBlurSizeIsCorrect(self):
            test_size = (5,3)
            test_surface = pygame.surface.Surface(test_size, pygame.SRCALPHA, 32)
            test_surface.set_at((2,1), (255,255,255,255))
            test_surface.set_at((3,1), (255,255,255,255))
            test_surface.set_at((2,2), (255,255,255,255))
            test_surface.set_at((3,2), (255,255,255,255))
            # if this radius gets too high, too little is blurred to the edge to show up
            radius = 3
            blur_buffer, blur_size = GetAlphaBlur(test_surface, radius)
            self.assertEqual(blur_size, (test_size[0] + radius*2, test_size[1] + radius*2))
            blur_surface = CreateSurfaceFromAlpha(blur_buffer, blur_size, (255,255,255,255))
            minRect = FindCropRect(test_surface)     
            minBlurRect = FindCropRect(blur_surface) 
            self.assertEqual((minRect.width + radius*2, minRect.height + radius*2), (minBlurRect.width, minBlurRect.height))

    def PrintPerformanceResults():
        defaultFont = pygame.font.Font(None, 24)
        period_surface = defaultFont.render(".", True, (255,255,255))
        perfTest(FindCropRect, period_surface)
        try:
            import psyco
        except:
            pass
        else:
            print "\npsychoing line scanning"
            psyco.bind(_LineScanBuffer)
            perfTest(FindCropRect, period_surface)

            print "\npsychoing called function"
            psyco.bind(FindCropRect)
            perfTest(FindCropRect, period_surface)

            print "\npsychoing test harness"
            psyco.bind(timeFunction)
            perfTest(FindCropRect, period_surface)
    
    pygame.font.init()
    #PrintPerformanceResults()
    unittest.main()
                